import {
    Component, Input, OnInit, ElementRef, HostBinding, AfterViewInit, ViewEncapsulation, OnDestroy, EventEmitter, Output
} from '@angular/core';
import { Observable, Subscription } from 'rxjs';
import { VgAPI } from '../core/services/vg-api';
import { VgControlsHidden } from './../core/services/vg-controls-hidden';

import { VgStates } from '../core/states/vg-states';
import { fromEvent } from 'rxjs';
import { VgFullscreenAPI } from '../core/services/vg-fullscreen-api';

@Component({
    selector: 'vg-controls',
    encapsulation: ViewEncapsulation.None,
    template: `<ng-content></ng-content>`,
    styles: [`
        vg-controls {
            position: absolute;
            display: flex;
            width: 100%;
            height: 50px;
            z-index: 300;
            bottom: 0;
            background-color: rgba(0, 0, 0, 0.5);
            -webkit-transition: bottom 1s;
            -khtml-transition: bottom 1s;
            -moz-transition: bottom 1s;
            -ms-transition: bottom 1s;
            transition: bottom 1s;
        }

        vg-controls.hide {
            bottom: -50px;
        }
    `]
})
export class VgControls implements OnInit, AfterViewInit, OnDestroy {
    elem: HTMLElement;
    target: any;

    @HostBinding('style.pointer-events') isAdsPlaying = 'initial';
    @HostBinding('class.hide') hideControls = true;

    @Input() vgFor: string;
    @Input() vgAutohide = false;
    @Input() vgAutohideTime = 3;
    @Input() vgIndex: Number;
    @Input() vgId: Number;
    @Input() vgIndexI: Number;
    @Input() showPlayIcon = 0;//0 为默认视频，播放完不显示播放按钮    1为评论视频，播放完显示按钮

    @Output() childbroadcastevent = new EventEmitter;

    private timer: any;
    @Input() isTry = false;
    mouseMove$: Observable<any>;
    touchStart$: Observable<any>;

    subscriptions: Subscription[] = [];
    timers;
    // @ts-ignore
    constructor(
        private API: VgAPI, 
        private ref: ElementRef, 
        private hidden: VgControlsHidden,
        public fsAPI: VgFullscreenAPI
        ) {
        this.elem = ref.nativeElement;
    }

    ngOnInit() {
        this.mouseMove$ = fromEvent(this.API.videogularElement, 'mousemove');
        this.subscriptions.push(this.mouseMove$.subscribe(this.show.bind(this)));

        this.touchStart$ = fromEvent(this.API.videogularElement, 'touchstart');
        this.subscriptions.push(this.touchStart$.subscribe(this.show.bind(this)));

        if (this.API.isPlayerReady) {
            this.onPlayerReady();
        }
        else {
            this.subscriptions.push(this.API.playerReadyEvent.subscribe(() => this.onPlayerReady()));
        }
    }

    onPlayerReady() {
        this.target = this.API.getMediaById(this.vgFor);

        this.subscriptions.push(this.target.subscriptions.play.subscribe(this.onPlay.bind(this)));
        this.subscriptions.push(this.target.subscriptions.pause.subscribe(this.onPause.bind(this)));
        this.subscriptions.push(this.target.subscriptions.ended.subscribe(this.onEnded.bind(this)));
        this.subscriptions.push(this.target.subscriptions.startAds.subscribe(this.onStartAds.bind(this)));
        this.subscriptions.push(this.target.subscriptions.endAds.subscribe(this.onEndAds.bind(this)));
    }

    ngAfterViewInit() {
        if (this.vgAutohide) {
            this.hide();
        }
        else {
            this.show();
        }
    }

    onPlay() {
        clearTimeout(this.timers);
        if (this.vgAutohide) {
            this.hide();
        }
        if (this.isTry) {
            this.timers = setInterval(() => {
                this.videoTime();
            }, 1000)
        }
        console.log('开始播放');
        console.log(this.subscriptions);
    }

    videoTime() {
        console.log(this.target.time.current);
        if (this.target.time.current > 20000) {
            this.target.pause();
            this.target.time.current = 20000;
            console.log(this.target.time.current);
            let percentage = this.target.time.current * 100 / this.target.time.total
            this.target.seekTime(percentage, true);
            this.changeShare()
        }
    }

    onPause() {
        clearTimeout(this.timer);
        this.hideControls = true;
        this.hidden.state(false);
        clearTimeout(this.timers);
    }

    onEnded() {
        // console.log('视频停止');
        // console.log(this.vgFor);
        if(this.showPlayIcon == 0){
            this.hidden.state(true);
            this.hideControls = true;
            this.changeShare();
        }else{
            // this.hideControls = false;
            this.hidden.state(false);
        }
        console.log('123');
        clearTimeout(this.timers);
    }

    changeShare() {
        let element = this.target;
        if (this.vgIndex) {
            let data = {
                id: this.vgFor,
                index: this.vgIndex,
                playId: this.vgId,
                indexI: this.vgIndexI
            }
            this.childbroadcastevent.emit(data);
        } else {
            this.childbroadcastevent.emit(this.vgFor);
        }
        //播放或试看结束,退出全屏
        this.fsAPI.toggleFullscreen(element,1);
        setTimeout(()=>{
            this.hidden.state(true);
        },1000)
    }


    onStartAds() {
        this.isAdsPlaying = 'none';
    }

    onEndAds() {
        this.isAdsPlaying = 'initial';
    }

    hide() {
        if (this.vgAutohide) {
            clearTimeout(this.timer);
            this.hideAsync();
        }
    }

    show() {
        clearTimeout(this.timer);
        this.hideControls = false;
        this.hidden.state(false);

        if (this.vgAutohide) {
            this.hideAsync();
        }
    }

    private hideAsync() {
        if (this.API.state === VgStates.VG_PLAYING) {
            this.timer = setTimeout(() => {
                this.hideControls = true;
                this.hidden.state(true);
            }, this.vgAutohideTime * 1000);
        }
    }

    ngOnDestroy() {
        this.subscriptions.forEach(s => s.unsubscribe());
    }
}
